/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  ** This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether 
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * COPYRIGHT(c) 2021 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <max7219.h>
#include <si5351.h>
#include <stdlib.h>
#include <math.h>
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc1;
ADC_HandleTypeDef hadc2;

I2C_HandleTypeDef hi2c1;

SPI_HandleTypeDef hspi1;

TIM_HandleTypeDef htim1;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_SPI1_Init(void);
static void MX_I2C1_Init(void);
static void MX_TIM1_Init(void);
static void MX_ADC1_Init(void);
static void MX_ADC2_Init(void);
/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

#define CH_VFO  0
#define CH_CAL  1
#define SI5351_CORRECTION 3894
#define BFO_FREQ 8997750
#define MIN_FREQ 7000000
#define MAX_FREQ 7200000
#define START_FREQ 7115000

#define BUTTON_DEBOUNCE_TIME_MS 200
typedef enum {
    BUTTON_STATUS_PRESSED = 0,
    BUTTON_STATUS_RELEASED = 1,
    BUTTON_STATUS_DEBOUNCE = 2,
} ButtonStatus_t;

void DislayLogo() {
    max7219_SendData(8, MAX7219_SYM_H);
    max7219_SendData(7, MAX7219_SYM_B);
    max7219_SendData(6, MAX7219_SYM_R);
    max7219_SendData(5, MAX7219_SYM_MINUS);
    max7219_SendData(4, MAX7219_SYM_4);
    max7219_SendData(3, MAX7219_SYM_0);
}

void DislayMonth() {
    max7219_SendData(8, MAX7219_SYM_0);
    max7219_SendData(7, MAX7219_SYM_9 | MAX7219_SEG_DP);
    max7219_SendData(6, MAX7219_SYM_2);
    max7219_SendData(5, MAX7219_SYM_0);
    max7219_SendData(4, MAX7219_SYM_2);
    max7219_SendData(3, MAX7219_SYM_3);
}

uint8_t NumberToSegments(int number) {
    switch(number) {
        case 0: return MAX7219_SYM_0;
        case 1: return MAX7219_SYM_1;
        case 2: return MAX7219_SYM_2;
        case 3: return MAX7219_SYM_3;
        case 4: return MAX7219_SYM_4;
        case 5: return MAX7219_SYM_5;
        case 6: return MAX7219_SYM_6;
        case 7: return MAX7219_SYM_7;
        case 8: return MAX7219_SYM_8;
        case 9: return MAX7219_SYM_9;
        default: return MAX7219_SYM_MINUS;
    }
}

void DisplayFreq(int32_t freq, bool lock, bool fast, bool dispDisabled) {
    if(dispDisabled) {
        max7219_Clean();
        max7219_SendData(3, MAX7219_SEG_DP);
        return;
    }

    max7219_SendData(8, NumberToSegments(freq / 1000000) | MAX7219_SEG_DP);
    max7219_SendData(7, NumberToSegments((freq / 100000) % 10));
    max7219_SendData(6, NumberToSegments((freq / 10000) % 10));
    max7219_SendData(5, NumberToSegments((freq / 1000) % 10) | MAX7219_SEG_DP);
    max7219_SendData(4, NumberToSegments((freq / 100) % 10));
  
    if(lock) {
        max7219_SendData(3, MAX7219_SYM_L);
    } else if(fast) {
        max7219_SendData(3, MAX7219_SYM_F);
    } else {
        max7219_SendData(3, NumberToSegments((freq / 10) % 10));
    }
}

void DisplaySWR(double swr) {
    max7219_SendData(8, MAX7219_SYM_S);
    max7219_SendData(7, MAX7219_SYM_W);
    max7219_SendData(6, MAX7219_SYM_R);
    max7219_SendData(5, MAX7219_SYM_BLANK);

    if(swr < 1) {
        max7219_SendData(4, MAX7219_SYM_MINUS | MAX7219_SEG_DP);
        max7219_SendData(3, MAX7219_SYM_MINUS);
    } else {
        max7219_SendData(4, NumberToSegments(((int)(swr * 10))/10) | MAX7219_SEG_DP);
        max7219_SendData(3, NumberToSegments(((int)(swr * 10))%10));
    }
}

void DisplayDisp() {
    max7219_SendData(8, MAX7219_SYM_MINUS);
    max7219_SendData(7, MAX7219_SYM_D);
    max7219_SendData(6, MAX7219_SYM_I);
    max7219_SendData(5, MAX7219_SYM_S);
    max7219_SendData(4, MAX7219_SYM_P);
    max7219_SendData(3, MAX7219_SYM_MINUS);
}

ButtonStatus_t buttonPressed(GPIO_TypeDef *GPIOx, uint16_t GPIO_Pin, uint32_t* lastPressed) {
    if(HAL_GPIO_ReadPin(GPIOx, GPIO_Pin) == GPIO_PIN_RESET) {
        uint32_t now = HAL_GetTick();
        if(now - *lastPressed > BUTTON_DEBOUNCE_TIME_MS) {
            *lastPressed = now;
            return BUTTON_STATUS_PRESSED;
        } else {
            return BUTTON_STATUS_DEBOUNCE;
        }
    }
    return BUTTON_STATUS_RELEASED;
}

ButtonStatus_t buttonLockPressed() {
    static uint32_t lastPressed = 0;
    return buttonPressed(GPIOA, GPIO_PIN_4, &lastPressed);
}

ButtonStatus_t buttonFastPressed() {
    static uint32_t lastPressed = 0;
    return buttonPressed(GPIOA, GPIO_PIN_3, &lastPressed);
}

ButtonStatus_t buttonDispPressed() {
    static uint32_t lastPressed = 0;
    return buttonPressed(GPIOA, GPIO_PIN_2, &lastPressed);
}

ButtonStatus_t buttonPttPressed() {
    static uint32_t lastPressed = 0;
    return buttonPressed(GPIOB, GPIO_PIN_11, &lastPressed);
}

void enableTx(bool enable) {
    HAL_GPIO_WritePin(GPIOB, GPIO_PIN_10, enable ? GPIO_PIN_SET : GPIO_PIN_RESET);
}

void changeFrequency(int32_t freq, bool lock, bool fast, bool dispDisabled) {
    static bool pllInitDone = false;
    static si5351PLLConfig_t pll_conf;
    si5351OutputConfig_t out_conf;
    int32_t Fvfo = freq + BFO_FREQ; // for LSB

    si5351_Calc(Fvfo, &pll_conf, &out_conf);

    if(!pllInitDone) {
        si5351_SetupPLL(SI5351_PLL_A, &pll_conf);
        si5351_EnableOutputs(1<<CH_VFO);
        pllInitDone = true;
    }

    si5351_SetupOutput(CH_VFO, SI5351_PLL_A, SI5351_DRIVE_STRENGTH_4MA, &out_conf);
    DisplayFreq(freq, lock, fast, dispDisabled);
}

void init() {
    enableTx(false);

    HAL_TIM_Encoder_Start(&htim1, TIM_CHANNEL_ALL);
    HAL_ADC_Start(&hadc1);
    HAL_ADC_Start(&hadc2);
    
    max7219_Init();
    max7219_SetIntensivity(15);
    max7219_TurnOn();

    si5351_Init(SI5351_CORRECTION);

/*
    // Code for determining the correction factor for Si5351
    max7219_SendData(8, MAX7219_SYM_C);
    max7219_SendData(7, MAX7219_SYM_A);
    max7219_SendData(6, MAX7219_SYM_L | MAX7219_SEG_DP);
    max7219_SendData(5, MAX7219_SYM_C);
    max7219_SendData(4, MAX7219_SYM_H);
    max7219_SendData(3, NumberToSegments(CH_CAL));

    si5351_Init(0);
    si5351PLLConfig_t pll_conf;
    si5351OutputConfig_t out_conf;
    int32_t Fclk = 10000000;

    si5351_Calc(Fclk, &pll_conf, &out_conf);
    si5351_SetupPLL(SI5351_PLL_A, &pll_conf);
    si5351_SetupOutput(CH_CAL, SI5351_PLL_A, SI5351_DRIVE_STRENGTH_4MA, &out_conf);
    si5351_EnableOutputs(1<<CH_CAL);

    while(1) {
        HAL_Delay(100);
    }
*/

    DislayLogo();
    HAL_Delay(1000);
    DislayMonth();
    HAL_Delay(1000);
    
    changeFrequency(START_FREQ, false, false, false);
}

int32_t targetFreq = START_FREQ;
int32_t prevCounter = 0;
bool lockMode = false;
bool fastMode = false;
bool dispDisabled = false;
bool transmitMode = false;
double prevSWRValue;
double lastShownSWRValue;
int32_t numSWRSamples;

void loop() {
    int32_t currCounter = __HAL_TIM_GET_COUNTER(&htim1);
    currCounter = 32767 - ((currCounter-1) & 0xFFFF) / 2;
    if(currCounter > 32768/2) {
        // convert ... 32766, 32767, 0, 1, 2 ... into:
        //               ... -2, -1, 0, 1, 2 ...
        // this simplifies `delta` calculation
        currCounter = currCounter-32768;
    }

    HAL_Delay(5);

    if(transmitMode) {
        double v_fwd, v_ref, ratio, swr;

        HAL_ADC_PollForConversion(&hadc1, HAL_MAX_DELAY);
        v_fwd = ((double)HAL_ADC_GetValue(&hadc1))*3.3/4096.0;

        if(v_fwd > 0.5) { /* transmitting */
            HAL_ADC_PollForConversion(&hadc2, HAL_MAX_DELAY);
            v_ref = ((double)HAL_ADC_GetValue(&hadc2))*3.3/4096.0;

            if(v_ref < 0.1) {
                v_ref = 0.0;
            }

            if(v_ref > 0.8*v_fwd) {
                /* can't display SWR >= 10.0 */
                swr = 9.5;
            } else {
                ratio = v_ref / v_fwd;
                swr = (1.0+ratio)/(1.0-ratio);
            }

            /* averaging */
            swr = (prevSWRValue*9 + swr)/10;
            prevSWRValue = swr;

            if(numSWRSamples < 10) {
                /* need to collect more data */
                numSWRSamples++;
            } else {
                if(fabs(lastShownSWRValue - swr) > 0.1) {
                    DisplaySWR(swr);
                    lastShownSWRValue = swr;
                }
            }
        }

        if(buttonPttPressed() == BUTTON_STATUS_RELEASED) {
            transmitMode = false;
            enableTx(false);
            changeFrequency(targetFreq, lockMode, fastMode, dispDisabled);
        } else {
            prevCounter = currCounter;
            return;
        }
    }

    if(buttonLockPressed() == BUTTON_STATUS_PRESSED) {
        lockMode = !lockMode;
        changeFrequency(targetFreq, lockMode, fastMode, dispDisabled);
    }

    if(lockMode) {
        prevCounter = currCounter;
        return;
    }

    if(buttonPttPressed() == BUTTON_STATUS_PRESSED) {
        transmitMode = true;
        DisplaySWR(-1);
        enableTx(true);
        prevCounter = currCounter;
        lastShownSWRValue = 0.0;
        prevSWRValue = 1.0;
        numSWRSamples = 0;
        return;
    }

    if(buttonFastPressed() == BUTTON_STATUS_PRESSED) {
        fastMode = !fastMode;
        changeFrequency(targetFreq, lockMode, fastMode, dispDisabled);
    }

    if(buttonDispPressed() == BUTTON_STATUS_PRESSED) {
        dispDisabled = !dispDisabled;
        changeFrequency(targetFreq, lockMode, fastMode, dispDisabled);
    }

    if(currCounter != prevCounter) {
        int32_t delta = prevCounter-currCounter;
        prevCounter = currCounter;

        // debounce or skip counter overflow
        if((delta > -10) && (delta < 10)) {
            if(fastMode) {
                targetFreq += delta * 1000;
                targetFreq -= targetFreq % 1000;
            } else {
                targetFreq += delta * 50;
            }

            if(targetFreq < MIN_FREQ) {
                targetFreq = MIN_FREQ;
            } else if(targetFreq > MAX_FREQ) {
                targetFreq = MAX_FREQ;
            }

            changeFrequency(targetFreq, lockMode, fastMode, dispDisabled);
        }
    }
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_SPI1_Init();
  MX_I2C1_Init();
  MX_TIM1_Init();
  MX_ADC1_Init();
  MX_ADC2_Init();
  /* USER CODE BEGIN 2 */
  init();

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    loop();
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */

  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /**Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /**Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_ADC;
  PeriphClkInit.AdcClockSelection = RCC_ADCPCLK2_DIV2;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief ADC1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC1_Init(void)
{

  /* USER CODE BEGIN ADC1_Init 0 */

  /* USER CODE END ADC1_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC1_Init 1 */

  /* USER CODE END ADC1_Init 1 */
  /**Common config 
  */
  hadc1.Instance = ADC1;
  hadc1.Init.ScanConvMode = ADC_SCAN_DISABLE;
  hadc1.Init.ContinuousConvMode = ENABLE;
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 1;
  if (HAL_ADC_Init(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }
  /**Configure Regular Channel 
  */
  sConfig.Channel = ADC_CHANNEL_0;
  sConfig.Rank = ADC_REGULAR_RANK_1;
  sConfig.SamplingTime = ADC_SAMPLETIME_1CYCLE_5;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC1_Init 2 */

  /* USER CODE END ADC1_Init 2 */

}

/**
  * @brief ADC2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC2_Init(void)
{

  /* USER CODE BEGIN ADC2_Init 0 */

  /* USER CODE END ADC2_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC2_Init 1 */

  /* USER CODE END ADC2_Init 1 */
  /**Common config 
  */
  hadc2.Instance = ADC2;
  hadc2.Init.ScanConvMode = ADC_SCAN_DISABLE;
  hadc2.Init.ContinuousConvMode = ENABLE;
  hadc2.Init.DiscontinuousConvMode = DISABLE;
  hadc2.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc2.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc2.Init.NbrOfConversion = 1;
  if (HAL_ADC_Init(&hadc2) != HAL_OK)
  {
    Error_Handler();
  }
  /**Configure Regular Channel 
  */
  sConfig.Channel = ADC_CHANNEL_1;
  sConfig.Rank = ADC_REGULAR_RANK_1;
  sConfig.SamplingTime = ADC_SAMPLETIME_1CYCLE_5;
  if (HAL_ADC_ConfigChannel(&hadc2, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC2_Init 2 */

  /* USER CODE END ADC2_Init 2 */

}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.ClockSpeed = 400000;
  hi2c1.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief SPI1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_SPI1_Init(void)
{

  /* USER CODE BEGIN SPI1_Init 0 */

  /* USER CODE END SPI1_Init 0 */

  /* USER CODE BEGIN SPI1_Init 1 */

  /* USER CODE END SPI1_Init 1 */
  /* SPI1 parameter configuration*/
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_1LINE;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_SOFT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_2;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN SPI1_Init 2 */

  /* USER CODE END SPI1_Init 2 */

}

/**
  * @brief TIM1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM1_Init(void)
{

  /* USER CODE BEGIN TIM1_Init 0 */

  /* USER CODE END TIM1_Init 0 */

  TIM_Encoder_InitTypeDef sConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};

  /* USER CODE BEGIN TIM1_Init 1 */

  /* USER CODE END TIM1_Init 1 */
  htim1.Instance = TIM1;
  htim1.Init.Prescaler = 0;
  htim1.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim1.Init.Period = 65535;
  htim1.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim1.Init.RepetitionCounter = 0;
  htim1.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  sConfig.EncoderMode = TIM_ENCODERMODE_TI1;
  sConfig.IC1Polarity = TIM_ICPOLARITY_RISING;
  sConfig.IC1Selection = TIM_ICSELECTION_DIRECTTI;
  sConfig.IC1Prescaler = TIM_ICPSC_DIV1;
  sConfig.IC1Filter = 0;
  sConfig.IC2Polarity = TIM_ICPOLARITY_RISING;
  sConfig.IC2Selection = TIM_ICSELECTION_DIRECTTI;
  sConfig.IC2Prescaler = TIM_ICPSC_DIV1;
  sConfig.IC2Filter = 0;
  if (HAL_TIM_Encoder_Init(&htim1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim1, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM1_Init 2 */

  /* USER CODE END TIM1_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_6, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_10, GPIO_PIN_RESET);

  /*Configure GPIO pins : PA2 PA3 PA4 */
  GPIO_InitStruct.Pin = GPIO_PIN_2|GPIO_PIN_3|GPIO_PIN_4;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : PA6 */
  GPIO_InitStruct.Pin = GPIO_PIN_6;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : PB10 */
  GPIO_InitStruct.Pin = GPIO_PIN_10;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pin : PB11 */
  GPIO_InitStruct.Pin = GPIO_PIN_11;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
